class DataList:
    #class 2 数据列表化，将单一的温度，升温速率和dadT生成列表
    def __init__(self,temperature, beta, dadT, unit = "c"):
        """
A class for storing thermal analysis data at a specific conversion level (α).

Parameters
----------
temperature : list[float]
    Temperature list corresponding to each heating rate point.
beta : list[float]
    Heating rate list (β), typically in K/min.
dadt : list[float]
    Reaction rate list (dα/dt), corresponding to the same temperatures.
unit : str, optional    
    Temperature unit indicator:
        - 'c' : data collected in Celsius
        - 'f' : data collected in Fahrenheit
        - 'k' : data already in Kelvin
    Default is 'c'.
    
Notes
-----
If the temperature is in Celsius or Fahrenheit, it will be converted to Kelvin automatically.
"""
        self.temperature = list(temperature)    # 对应温度列表
        if unit is not None and unit.lower() == "c":
            self.temperature= [ t + 273.15 for t in self.temperature]
        if unit is not None and unit.lower() == "f":
            self.temperature= [ (t - 32) / 1.8 + 273.15 for t in self.temperature]
        elif unit not in (None, "c", "C", "k", "K",'f','F'):
            raise ValueError("unit 参数只能是 'c' 或 'k'")
        self.beta = list(beta)  #对应升温速率列表
        self.dadT = list(dadT)  #对应da/dT列表



import numpy as np
import scipy.optimize as sp
def r_square_xy(x,y):
    # function 1 
    # r_square 用于输出k, b, r^2
    """
Calculate the correlation coefficient (R²) of the regression line
for arrays x and y.

Formula:
    R² = SSR / SST = 1 - SSE / SST
where:
    SSR — Regression Sum of Squares
    SST — Total Sum of Squares
    SSE — Error Sum of Squares

The regression line is expressed as:
    ŷ = k * x + b

Returns
-------
k : float
    Slope of the regression line.
b : float
    Intercept of the regression line.
r2 : float
    Coefficient of determination (R²).
"""
    x = np.array(x)
    y = np.array(y)
    k,b  = np.polyfit(x, y, deg = 1)    # 一阶线性拟合
    y_hat = k * np.array(x) + b
    y_bar = sum(y) / len(y)
    ss_residual = np.sum((y - y_hat) ** 2)    # 总平方和
    ss_total = np.sum((y - y_bar) ** 2) # 残差平方和
    r_2 = 1 - ss_residual / ss_total
    return k, b, r_2

class KineticAnalysis():
    """
A class for performing kinetic analysis.

Parameters
----------
alpha_list : list of float
    Conversion degrees, e.g. [α1, α2, α3, ...].

data_objects : list of DataList
    Each DataList contains corresponding temperature, heating rate, 
    and dα/dT data for a given experimental condition.

Details
-------
temperature : list of tuple(float)
    Temperature datasets for each heating rate, e.g. 
    [(T11, T12, T13, ... T1n), (T21, T22, T23, ... T2n), ...].

beta : list of float
    Heating rate list, e.g. [b1, b2, b3, ..., bn].

dadT : list of tuple(float)
    dα/dT datasets corresponding to temperature, e.g. 
    [(dadT11, dadT12, dadT13, ... dadT1n), 
     (dadT21, dadT22, dadT23, ... dadT2n), ...].
"""
    #class3 热动力学分析        
    # 数据结构：alpha = [a1,a2,a3],     
    # temperatrue = [(T11, T12, T13),(T21, T22, T23),(T31, T32, T33)] 
    # beta = [(b11, b12, b13),(b21, b22, b23),(b31, b32, b33)]
    # dadT = for friedman method only
    # 温度，升温速率和dadT是通过class DataList生成的小列表再热分析中嵌入大列表
    R = 8.314 # gas constant 8.314 J/mol·K
    def __init__(self, alpha_list, data_objects):
        self.alpha_list = list(alpha_list)
        self.objects_data = list(data_objects)
    
    def fwo_ea(self, return_data = False):
        self.results = {}   # 计算的结果为一个字典
        self.raw_data = {}
        #ln(beta) = const - 1.052*Ea/RT
        for alpha, data in zip(self.alpha_list,self.objects_data):
            log_beta = [np.log(beta) for beta in data.beta]
            data_temp = [1 / temp for temp in data.temperature]
            k, b, r2 = r_square_xy(data_temp,log_beta)
            ea = - self.R / 1000 * k / 1.052
            self.results[alpha] = {'Ea = ': ea,
                            'k = ': k,
                            'b = ': b,
                            'r^2 = ': r2}
            # e.g., results(0.1: {'Ea = ': 114.514,'k = ': 1919, 'b = ': 810, 'r^2 = ': 0.9527})
            # e.g., unit of Ea is kJ/mol
            self.raw_data[alpha] = {
               'x': [float(temp) for temp in data_temp],
                'y': [float(beta) for beta in log_beta]
                }
        if return_data:
            return self.results, self.raw_data
        else:
            return self.results
    
    def kas_ea(self, return_data = False):
        self.results = {}   # 计算的结果为一个字典
        self.raw_data = {}
        for alpha, data in zip(self.alpha_list,self.objects_data):
            log_beta = [np.log(beta / temp ** 2) for beta, temp\
                         in zip(data.beta, data.temperature)]
            data_temp = [1 / temp for temp in data.temperature]
            k, b, r2 = r_square_xy(data_temp,log_beta)
            ea = - self.R / 1000 * k
            self.results[alpha] = {'Ea = ': ea,
                            'k = ': k,
                            'b = ': b,
                            'r^2 = ': r2}
            # e.g., results(0.2: {'Ea = ': 114.514,'k = ': 1919, 'b = ': 810, 'r^2 = ': 0.9527})
            # e.g., unit of Ea is kJ/mol
            self.raw_data[alpha] = {
               'x': [float(temp) for temp in data_temp],
                'y': [float(beta) for beta in log_beta]
                }
        if return_data:
            return self.results, self.raw_data
        else:
            return self.results
    
    def starink_ea(self, return_data = False):
        self.results = {}   # 计算的结果为一个字典
        self.raw_data = {}
        for alpha, data in zip(self.alpha_list, self.objects_data):
            log_beta = [np.log(beta / temperature ** 1.92 ) for beta, temperature \
                         in zip(data.beta,data.temperature)]
            data_temp = [1 / temp for temp in data.temperature]
            k, b, r2 = r_square_xy(data_temp,log_beta)
            ea = - self.R / 1000 * k / 1.0008
            self.results[alpha] = {'Ea = ': ea,
                            'k = ': k,
                            'b = ': b,
                            'r^2 = ': r2}
            self.raw_data[alpha] = {
            'x': [float(temp) for temp in data_temp],
            'y': [float(beta) for beta in log_beta]
            }
        if return_data:
            return self.results, self.raw_data
        else:
            return self.results
    
    def friedman_ea(self, return_data = False):
        self.results = {}   # 计算的结果为一个字典
        self.raw_data = {}
        for alpha, data in zip(self.alpha_list, self.objects_data):
            log_beta = [np.log(beta * dadT) for beta, dadT \
                         in zip(data.beta, data.dadT)]
            data_temp = [1 / temp for temp in data.temperature]
            k, b, r2 = r_square_xy(data_temp, log_beta)
            ea = - self.R / 1000 * k
            self.results[alpha] = {'Ea = ': ea,
                            'k = ': k,
                            'b = ': b,
                            'r^2 = ': r2}
            # e.g., results(0.7 :{'Ea = ': 114.514,'k = ': 1919, 'b = ': 810, 'r^2 = ': 0.9527})
            # e.g., unit of Ea is kJ/mol
            self.raw_data[alpha] = {
            'x': [float(temp) for temp in data_temp],
            'y': [float(beta) for beta in log_beta]
            }
        if return_data:
            return self.results, self.raw_data
        else:
            return self.results
        
    def vyazovkin_ea(self, return_data = False):
        self.results = dict()
        self.raw_data = dict()
        # （cai近似式）        
        def I(E, T):
            R = self.R / 1000  #  kJ/mol
            return (R * T ** 2 / E) * ((E + 0.66691 * R * T) / (E + 2.64943 * R * T)) * np.exp(-E / (R * T))
        
        for alpha, data in zip(self.alpha_list, self.objects_data):
            def Phi(E):
                Phi = 0.0
                for i in range(len(data.beta)):
                    for j in range(len(data.beta)):
                        if j == i:
                            continue
                        else:
                            Phi += (I(E, data.temperature[i]) / data.beta[i]) / (I(E, data.temperature[j]) / data.beta[j])
                return Phi            

            Phi_min  = sp.minimize_scalar(Phi, bounds = (10, 1000),method = 'bounded')
            self.results[alpha] = {'Ea = ': float(Phi_min.x)}
            a = float(Phi_min.x)
            self.raw_data[alpha] = {'x': np.linspace((a - 50),(a + 50), 100),
                             'y': [float(Phi(i)) for i in np.linspace((a - 50),(a + 50), 100)]
                            }
        if return_data:
            return self.results, self.raw_data
        else:
            return self.results


import numpy as np
import matplotlib.pyplot as plt   
from .thermoanalysis import KineticAnalysis
class FittingPlot:
    """
A class for plotting scatter data and fitted regression curves 
based on kinetic analysis results.

Parameters
----------
alpha_list : list[float]
List of conversion fractions (α values).
data_objects : list[DataList]
List of `DataList` instances, each containing temperature, 
heating rate (β), and reaction rate (dα/dt) data for a specific α.
analysis : KineticAnalysis
An instance of `KineticAnalysis` used to perform the fitting 
and provide the regression results.
"""
    def __init__(self, alpha_list, data_objects, analysis: KineticAnalysis):
        self.alpha_list = list(alpha_list)
        self.objects_data = list(data_objects)
        self.analysis = analysis

    def fwoplot(self):
        results = self.analysis.fwo_ea(return_data = True)
        figure = plt.figure(figsize= (8,6))
        axes = plt.subplot(1,1,1)
        axes.set_xlabel("1/T",fontsize = 14, fontname = 'arial')
        axes.set_ylabel('ln(β)',fontsize = 14, fontname = 'arial')
        scatterdata = results[1]
        plotdata = results[0]
        for key1,key2 in zip(scatterdata,plotdata):
            x = scatterdata[key1].get('x')
            y = scatterdata[key1].get('y')
            x_min, x_max = np.min(x), np.max(x)
            x_plot = np.linspace(x_min, x_max,10)
            axes.scatter(x,y)
            axes.plot(x_plot, plotdata[key2].get('k = ') * x_plot + plotdata[key2].get('b = ' ))
            plt.draw()   # 刷新绘图
            plt.pause(0.2)  # 暂停 0.3 秒，看清楚每条线
        plt.ioff()  # 关闭交互模式
        plt.show()

    def kasplot(self):
        results = self.analysis.kas_ea(return_data = True)
        figure = plt.figure(figsize= (8,6))
        axes = plt.subplot(1,1,1)
        axes.set_xlabel("1/T",fontsize = 14, fontname = 'arial')
        axes.set_ylabel('ln(β/T$^{2}$)',fontsize = 14, fontname = 'arial')
        scatterdata = results[1]
        plotdata = results[0]
        for key1,key2 in zip(scatterdata,plotdata):
            x = scatterdata[key1].get('x')
            y = scatterdata[key1].get('y')
            x_min, x_max = np.min(x), np.max(x)
            x_plot = np.linspace(x_min, x_max,10)
            axes.scatter(x,y)
            axes.plot(x_plot, plotdata[key2].get('k = ') * x_plot + plotdata[key2].get('b = ' ))
            plt.draw()   # 刷新绘图
            plt.pause(0.2)  # 暂停 0.3 秒，看清楚每条线
        plt.ioff()  # 关闭交互模式
        plt.show()
    
    def starinkplot(self):
        results = self.analysis.starink_ea(return_data = True)
        figure = plt.figure(figsize= (8,6))
        axes = plt.subplot(1,1,1)
        axes.set_xlabel("1/T",fontsize = 14, fontname = 'arial')
        axes.set_ylabel('ln(β/T$^{1.92}$)',fontsize = 14, fontname = 'arial')
        scatterdata = results[1]
        plotdata = results[0]
        for key1,key2 in zip(scatterdata,plotdata):
            x = scatterdata[key1].get('x')
            y = scatterdata[key1].get('y')
            x_min, x_max = np.min(x), np.max(x)
            x_plot = np.linspace(x_min, x_max,10)
            axes.scatter(x,y)
            axes.plot(x_plot, plotdata[key2].get('k = ') * x_plot + plotdata[key2].get('b = ' ))
            plt.draw()   # 刷新绘图
            plt.pause(0.2)  # 暂停 0.3 秒，看清楚每条线
        plt.ioff()  # 关闭交互模式
        plt.show()

    def friedmanplot(self):
        results = self.analysis.friedman_ea(return_data = True)
        figure = plt.figure(figsize= (8,6))
        axes = plt.subplot(1,1,1)
        axes.set_xlabel("1/T",fontsize = 14, fontname = 'arial')
        axes.set_ylabel('ln(da/dt)',fontsize = 14, fontname = 'arial')
        scatterdata = results[1]
        plotdata = results[0]
        for key1,key2 in zip(scatterdata,plotdata):
            x = scatterdata[key1].get('x')
            y = scatterdata[key1].get('y')
            x_min, x_max = np.min(x), np.max(x)
            x_plot = np.linspace(x_min, x_max,10)
            axes.scatter(x,y)
            axes.plot(x_plot, plotdata[key2].get('k = ') * x_plot + plotdata[key2].get('b = ' ))
            plt.draw()   # 刷新绘图
            plt.pause(0.2)  # 暂停 0.3 秒，看清楚每条线
        plt.ioff()  # 关闭交互模式
        plt.show()

    def vyazovkinplot(self):
        results = self.analysis.vyazovkin_ea (return_data = True)
        axes = plt.subplot(1,1,1)
        axes.set_xlabel("E(kJ/mol)",fontsize = 14, fontname = 'arial')
        axes.set_ylabel('$\Phi$(E)',fontsize = 14, fontname = 'arial')
        plotdata = results[1]
        for key in plotdata:
            x = plotdata[key].get('x')
            y = plotdata[key].get('y')
            axes.plot(x,y)
            plt.draw()   # 刷新绘图
            plt.pause(0.2)  # 暂停 0.3 秒，看清楚每条线
        plt.draw()
        plt.show()


# thermohi/__init__.py
from .datalistio import DataList
from .thermoanalysis import KineticAnalysis, r_square_xy
from .plotting import FittingPlot
__version__ = '0.2.1'
__author__ = "Hikari Quicklime"
__email__ = "897237104@qq.com"
__all__ = ["DataList", "KineticAnalysis", "r_square_xy","FittingPlot"]


import pytest
from thermohi_hikari.datalistio import DataList
from thermohi_hikari.thermoanalysis import KineticAnalysis, r_square_xy
from thermohi_hikari.plotting import FittingPlot


# 验证温度转换功能
def test_cel_to_kelvin():
    data = DataList(temperature = [0, 100], beta = [5, 10],dadT = [0.01, 0.02],unit= 'c')
    assert pytest.approx(data.temperature) == [273.15, 373.15]

def test_fahrenheit_to_kelvin():
    data = DataList(temperature = [32, 212], beta = [5, 10],dadT = [0.01, 0.02],unit= 'f')
    assert pytest.approx(data.temperature) == [273.15, 373.15]

def test_kelvin_directly():
    data = DataList(temperature = [273.15, 373.15], beta = [5, 10],dadT = [0.01, 0.02],unit= 'k')
    assert pytest.approx(data.temperature) == [273.15, 373.15]

def test_invalid_unit_raises():
    with pytest.raises(ValueError):
        DataList(
            temperature=[100, 200],
            beta=[10, 20],
            dadT=[0.1, 0.2],
            unit="xyz"
        )
# 验证r^2计算程序
def test_r_square():
    x = [1, 2, 3, 4, 5]
    y_true = [2, 4, 6, 8, 10]
    y_pred1 = [2.2, 3.8, 6.1, 7.9, 10.2]
    y_pred2 = [3, 5, 5, 9, 7]
     # 完美线性
    k, b, r2 = r_square_xy(x, y_true)
    assert k == pytest.approx(2.0)
    assert b == pytest.approx(0.0)
    assert r2 == pytest.approx(1.0)

    # 轻微偏差
    k, b, r2 = r_square_xy(x, y_pred1)
    assert k == pytest.approx(2.01, rel=1e-2)
    assert b == pytest.approx(0.01, rel=1e-2)
    assert r2 == pytest.approx(0.9968, rel=1e-3)

    # 偏差较大
    k, b, r2 = r_square_xy(x, y_pred2)
    assert k == pytest.approx(1.2, rel=1e-1)
    assert b == pytest.approx(2.2, rel=1e-1)
    assert r2 == pytest.approx(0.6923, rel=1e-2)

# 测试活化能：
alpha = [0.1, 0.5, 0.9]
data_object = [DataList([241.9, 251.85, 263.5, 270.9], [5,10,20,30], 
                        [0.00366, 0.00358, 0.00353, 0.00348], unit='c'),
               DataList([291.25, 302.3, 314.3, 322.45], [5,10,20,30], 
                        [0.01318, 0.01297, 0.01292, 0.01274], unit='c'),
               DataList([321.9, 333.55, 345.6, 354.2], [5,10,20,30],
                        [0.00942, 0.00927, 0.00925, 0.00914], unit='c')]
def test_fwo():
    analysis = KineticAnalysis(alpha, data_object)
    result = analysis.fwo_ea(return_data = False)
    a, b, c= [result[i].get('Ea = ') for i in alpha] 
    assert a == pytest.approx(136.5535)
    assert b == pytest.approx(153.1332)
    assert c == pytest.approx(164.8637)

def test_kas():
    analysis = KineticAnalysis(alpha, data_object)
    result = analysis.kas_ea(return_data = False)
    a, b, c= [result[i].get('Ea = ') for i in alpha] 
    assert a == pytest.approx(134.8546)
    assert b == pytest.approx(151.4584)
    assert c == pytest.approx(163.2806)

def test_starink():
    analysis = KineticAnalysis(alpha, data_object)
    result = analysis.starink_ea(return_data = False)
    a, b, c= [result[i].get('Ea = ') for i in alpha] 
    assert a == pytest.approx(135.0985)
    assert b == pytest.approx(151.7226)
    assert c == pytest.approx(163.5560)

def test_friedman():
    analysis = KineticAnalysis(alpha, data_object)
    result = analysis.friedman_ea(return_data = False)
    a, b, c= [result[i].get('Ea = ') for i in alpha] 
    assert a == pytest.approx(139.7619)
    assert b == pytest.approx(158.3342)
    assert c == pytest.approx(170.8158)



from setuptools import setup, find_packages

setup(
    name="thermohipy",
    version='0.2.1',
    packages=find_packages(),
    install_requires=[
        'numpy',
        'matplotlib',
        'scipy'
    ],
    python_requires='>=3.9',
    author='Hikari Quicklime',
    description='Thermal analysis toolkit for non-isothermal kinetic analysis (FWO, KAS, Starink, etc.)',
    url='https://github.com/QuicklimeHikari/thermohi'
)